@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA	02111-1301	USA
  *
  */
 
@@ -734,14 +734,14 @@ _get_details_resp_add_privacy_item(NMUser *user, NMERR_T ret_code,
 		if (allowed) {
 
 			if (!g_slist_find_custom(gc->account->permit,
-									 display_id, (GCompareFunc)nm_utf8_strcasecmp)) {
+									 display_id, (GCompareFunc)purple_utf8_strcasecmp)) {
 				purple_privacy_permit_add(gc->account, display_id, TRUE);
 			}
 
 		} else {
 
 			if (!g_slist_find_custom(gc->account->permit,
-									 display_id, (GCompareFunc)nm_utf8_strcasecmp)) {
+									 display_id, (GCompareFunc)purple_utf8_strcasecmp)) {
 				purple_privacy_deny_add(gc->account, display_id, TRUE);
 			}
 		}
@@ -782,7 +782,7 @@ _create_privacy_item_deny_resp_cb(NMUser *user, NMERR_T ret_code,
 		if (display_id) {
 
 			if (!g_slist_find_custom(gc->account->deny,
-									 display_id, (GCompareFunc)nm_utf8_strcasecmp)) {
+									 display_id, (GCompareFunc)purple_utf8_strcasecmp)) {
 
 				purple_privacy_deny_add(gc->account, display_id, TRUE);
 			}
@@ -834,7 +834,7 @@ _create_privacy_item_permit_resp_cb(NMUser *user, NMERR_T ret_code,
 
 			if (!g_slist_find_custom(gc->account->permit,
 									 display_id,
-									 (GCompareFunc)nm_utf8_strcasecmp)) {
+									 (GCompareFunc)purple_utf8_strcasecmp)) {
 
 				purple_privacy_permit_add(gc->account, display_id, TRUE);
 			}
@@ -1412,7 +1412,7 @@ _sync_privacy_lists(NMUser *user)
 			name =(char *)node->data;
 
 		if (!g_slist_find_custom(gc->account->permit,
-								 name, (GCompareFunc)nm_utf8_strcasecmp)) {
+								 name, (GCompareFunc)purple_utf8_strcasecmp)) {
 			purple_privacy_permit_add(gc->account, name , TRUE);
 		}
 	}
@@ -1425,7 +1425,7 @@ _sync_privacy_lists(NMUser *user)
 			name =(char *)node->data;
 
 		if (!g_slist_find_custom(gc->account->deny,
-								 name, (GCompareFunc)nm_utf8_strcasecmp)) {
+								 name, (GCompareFunc)purple_utf8_strcasecmp)) {
 			purple_privacy_deny_add(gc->account, name, TRUE);
 		}
 	}
@@ -1436,7 +1436,7 @@ _sync_privacy_lists(NMUser *user)
 		dn = nm_lookup_dn(user, (char *)node->data);
 		if (dn != NULL &&
 			!g_slist_find_custom(user->allow_list,
-								 dn, (GCompareFunc)nm_utf8_strcasecmp)) {
+								 dn, (GCompareFunc)purple_utf8_strcasecmp)) {
 			rem_list = g_slist_append(rem_list, node->data);
 		}
 	}
@@ -1453,7 +1453,7 @@ _sync_privacy_lists(NMUser *user)
 		dn = nm_lookup_dn(user, (char *)node->data);
 		if (dn != NULL &&
 			!g_slist_find_custom(user->deny_list,
-								 dn, (GCompareFunc)nm_utf8_strcasecmp)) {
+								 dn, (GCompareFunc)purple_utf8_strcasecmp)) {
 			rem_list = g_slist_append(rem_list, node->data);
 		}
 	}
@@ -3268,7 +3268,7 @@ novell_set_permit_deny(PurpleConnection *gc)
 						name = nm_user_record_get_display_id(user_record);
 
 						if (!g_slist_find_custom(gc->account->permit,
-												 name, (GCompareFunc)nm_utf8_strcasecmp)) {
+												 name, (GCompareFunc)purple_utf8_strcasecmp)) {
 							purple_privacy_permit_add(gc->account, name , TRUE);
 						}
 					}
@@ -3282,7 +3282,7 @@ novell_set_permit_deny(PurpleConnection *gc)
 						name = nm_user_record_get_display_id(user_record);
 
 						if (!g_slist_find_custom(user->allow_list,
-												 dn, (GCompareFunc)nm_utf8_strcasecmp)) {
+												 dn, (GCompareFunc)purple_utf8_strcasecmp)) {
 							rc = nm_send_create_privacy_item(user, dn, TRUE,
 															 _create_privacy_item_deny_resp_cb,
 															 g_strdup(dn));
@@ -3310,7 +3310,7 @@ novell_set_permit_deny(PurpleConnection *gc)
 						name = nm_user_record_get_display_id(user_record);
 
 						if (!g_slist_find_custom(gc->account->deny,
-												 name, (GCompareFunc)nm_utf8_strcasecmp)) {
+												 name, (GCompareFunc)purple_utf8_strcasecmp)) {
 							purple_privacy_deny_add(gc->account, name , TRUE);
 						}
 					}
@@ -3325,7 +3325,7 @@ novell_set_permit_deny(PurpleConnection *gc)
 						name = nm_user_record_get_display_id(user_record);
 
 						if (!g_slist_find_custom(user->deny_list,
-												 dn, (GCompareFunc)nm_utf8_strcasecmp)) {
+												 dn, (GCompareFunc)purple_utf8_strcasecmp)) {
 							rc = nm_send_create_privacy_item(user, dn, FALSE,
 															 _create_privacy_item_deny_resp_cb,
 															 g_strdup(name));
@@ -3358,7 +3358,7 @@ novell_set_permit_deny(PurpleConnection *gc)
 				contact = nm_folder_get_contact(user->root_folder, i);
 				dn = nm_contact_get_dn(contact);
 				if (dn && !g_slist_find_custom(user->allow_list,
-											   dn, (GCompareFunc)nm_utf8_strcasecmp))
+											   dn, (GCompareFunc)purple_utf8_strcasecmp))
 				{
 					rc = nm_send_create_privacy_item(user, dn, TRUE,
 													 _create_privacy_item_deny_resp_cb,
@@ -3377,7 +3377,7 @@ novell_set_permit_deny(PurpleConnection *gc)
 					contact = nm_folder_get_contact(folder, j);
 					dn = nm_contact_get_dn(contact);
 					if (dn && !g_slist_find_custom(user->allow_list,
-												   dn, (GCompareFunc)nm_utf8_strcasecmp))
+												   dn, (GCompareFunc)purple_utf8_strcasecmp))
 					{
 						rc = nm_send_create_privacy_item(user, dn, TRUE,
 														 _create_privacy_item_deny_resp_cb,
